@@ -1,5 +1,5 @@
 /* ncmpc (Ncurses MPD Client)
- * (c) 2004-2019 The Music Player Daemon Project
+ * (c) 2004-2020 The Music Player Daemon Project
  * Project homepage: http://musicpd.org
  *
  * This program is free software; you can redistribute it and/or modify
@@ -84,30 +84,28 @@ public:
 
 static void
 screen_file_load_list(struct mpdclient *c, const char *current_path,
-		      FileList *filelist)
+		      FileList &filelist) noexcept
 {
 	auto *connection = c->GetConnection();
 	if (connection == nullptr)
 		return;
 
 	mpd_send_list_meta(connection, current_path);
-	filelist->Receive(*connection);
+	filelist.Receive(*connection);
 
 	if (c->FinishCommand())
-		filelist->Sort();
+		filelist.Sort();
 }
 
 void
 FileBrowserPage::Reload(struct mpdclient &c)
 {
-	delete filelist;
-
-	filelist = new FileList();
+	filelist = std::make_unique<FileList>();
 	if (!current_path.empty())
 		/* add a dummy entry for ./.. */
 		filelist->emplace_back(nullptr);
 
-	screen_file_load_list(&c, current_path.c_str(), filelist);
+	screen_file_load_list(&c, current_path.c_str(), *filelist);
 
 	lw.SetLength(filelist->size());
 
@@ -121,7 +119,7 @@ FileBrowserPage::ChangeDirectory(struct mpdclient &c, std::string &&new_path)
 
 	Reload(c);
 
-	screen_browser_sync_highlights(filelist, &c.playlist);
+	screen_browser_sync_highlights(*filelist, c.playlist);
 
 	lw.Reset();
 
@@ -315,7 +313,7 @@ FileBrowserPage::Update(struct mpdclient &c, unsigned events) noexcept
 		      | MPD_IDLE_QUEUE
 #endif
 		      )) {
-		screen_browser_sync_highlights(filelist, &c.playlist);
+		screen_browser_sync_highlights(*filelist, c.playlist);
 		SetDirty();
 	}
 }
@@ -345,7 +343,7 @@ FileBrowserPage::OnCommand(struct mpdclient &c, Command cmd)
 
 	case Command::SCREEN_UPDATE:
 		Reload(c);
-		screen_browser_sync_highlights(filelist, &c.playlist);
+		screen_browser_sync_highlights(*filelist, c.playlist);
 		return false;
 
 	default:
