@@ -1,5 +1,5 @@
 /* ncmpc (Ncurses MPD Client)
- * (c) 2004-2019 The Music Player Daemon Project
+ * (c) 2004-2020 The Music Player Daemon Project
  * Project homepage: http://musicpd.org
  *
  * This program is free software; you can redistribute it and/or modify
@@ -48,25 +48,31 @@
 #define HIGHLIGHT  (0x01)
 #endif
 
-FileListPage::~FileListPage() noexcept
+FileListPage::FileListPage(ScreenManager &_screen, WINDOW *_w,
+			   Size size,
+			   const char *_song_format) noexcept
+	:ListPage(_w, size),
+	 screen(_screen),
+	 song_format(_song_format)
 {
-	delete filelist;
 }
 
+FileListPage::~FileListPage() noexcept = default;
+
 #ifndef NCMPC_MINI
 
 /* sync highlight flags with playlist */
 void
-screen_browser_sync_highlights(FileList *fl, const MpdQueue *playlist)
+screen_browser_sync_highlights(FileList &fl, const MpdQueue &playlist) noexcept
 {
-	for (unsigned i = 0; i < fl->size(); ++i) {
-		auto &entry = (*fl)[i];
+	for (unsigned i = 0; i < fl.size(); ++i) {
+		auto &entry = fl[i];
 		const auto *entity = entry.entity;
 
 		if (entity != nullptr && mpd_entity_get_type(entity) == MPD_ENTITY_TYPE_SONG) {
 			const auto *song = mpd_entity_get_song(entity);
 
-			if (playlist->ContainsUri(mpd_song_get_uri(song)))
+			if (playlist.ContainsUri(mpd_song_get_uri(song)))
 				entry.flags |= HIGHLIGHT;
 			else
 				entry.flags &= ~HIGHLIGHT;
