@@ -1,5 +1,5 @@
 /* ncmpc (Ncurses MPD Client)
- * (c) 2004-2019 The Music Player Daemon Project
+ * (c) 2004-2020 The Music Player Daemon Project
  * Project homepage: http://musicpd.org
  *
  * This program is free software; you can redistribute it and/or modify
@@ -30,13 +30,17 @@
 #include "screen_utils.hxx"
 #include "FileListPage.hxx"
 #include "filelist.hxx"
-#include "util/Macros.hxx"
+
+#include <iterator>
 
 #include <string.h>
 
 enum {
 	SEARCH_URI = MPD_TAG_COUNT + 100,
-	SEARCH_ARTIST_TITLE
+#if LIBMPDCLIENT_CHECK_VERSION(2,10,0)
+	SEARCH_MODIFIED,
+#endif
+	SEARCH_ARTIST_TITLE,
 };
 
 static constexpr struct {
@@ -64,6 +68,11 @@ search_get_tag_id(const char *name)
 	    strcasecmp(name, _("file")) == 0)
 		return SEARCH_URI;
 
+#if LIBMPDCLIENT_CHECK_VERSION(2,10,0)
+	if (strcasecmp(name, "modified") == 0)
+		return SEARCH_MODIFIED;
+#endif
+
 	for (unsigned i = 0; search_tag[i].name != nullptr; ++i)
 		if (strcasecmp(search_tag[i].name, name) == 0 ||
 		    strcasecmp(search_tag[i].localname, name) == 0)
@@ -72,12 +81,12 @@ search_get_tag_id(const char *name)
 	return -1;
 }
 
-typedef struct {
+struct SearchMode {
 	enum mpd_tag_type table;
 	const char *label;
-} search_type_t;
+};
 
-static constexpr search_type_t mode[] = {
+static constexpr SearchMode mode[] = {
 	{ MPD_TAG_TITLE, N_("Title") },
 	{ MPD_TAG_ARTIST, N_("Artist") },
 	{ MPD_TAG_ALBUM, N_("Album") },
@@ -95,6 +104,11 @@ static const char *const help_text[] = {
 	"",
 	"Advanced  -  <tag>:<search term> [<tag>:<search term>...]",
 	"		Example: artist:radiohead album:pablo honey",
+#if LIBMPDCLIENT_CHECK_VERSION(2,10,0)
+	"		Example: modified:14d (units: s, M, h, d, m, y)",
+#else
+	"		(\"modified:\" requires libmpdclient 2.10)",
+#endif
 	"",
 	"		Available tags: artist, album, title, track,",
 	"		name, genre, date composer, performer, comment, file",
@@ -108,17 +122,17 @@ class SearchPage final : public FileListPage {
 	std::string pattern;
 
 public:
-	SearchPage(ScreenManager &_screen, WINDOW *_w, Size size)
+	SearchPage(ScreenManager &_screen, WINDOW *_w, Size size) noexcept
 		:FileListPage(_screen, _w, size,
 			      !options.search_format.empty()
 			      ? options.search_format.c_str()
 			      : options.list_format.c_str()) {
 		lw.DisableCursor();
-		lw.SetLength(ARRAY_SIZE(help_text));
+		lw.SetLength(std::size(help_text));
 	}
 
 private:
-	void Clear(bool clear_pattern);
+	void Clear(bool clear_pattern) noexcept;
 	void Reload(struct mpdclient &c);
 	void Start(struct mpdclient &c);
 
@@ -136,7 +150,7 @@ public:
 	/* virtual methods from class ListText */
 	const char *GetListItemText(char *buffer, size_t size,
 				    unsigned idx) const noexcept override {
-		assert(idx < ARRAY_SIZE(help_text));
+		assert(idx < std::size(help_text));
 
 		if (idx == 0) {
 			snprintf(buffer, size,
@@ -160,11 +174,10 @@ public:
 };
 
 void
-SearchPage::Clear(bool clear_pattern)
+SearchPage::Clear(bool clear_pattern) noexcept
 {
 	if (filelist) {
-		delete filelist;
-		filelist = new FileList();
+		filelist = std::make_unique<FileList>();
 		lw.SetLength(0);
 	}
 	if (clear_pattern)
@@ -173,11 +186,10 @@ SearchPage::Clear(bool clear_pattern)
 	SetDirty();
 }
 
-static FileList *
+static std::unique_ptr<FileList>
 search_simple_query(struct mpd_connection *connection, bool exact_match,
 		    int table, const char *local_pattern)
 {
-	FileList *list;
 	const LocaleToUtf8 filter_utf8(local_pattern);
 
 	if (table == SEARCH_ARTIST_TITLE) {
@@ -197,15 +209,16 @@ search_simple_query(struct mpd_connection *connection, bool exact_match,
 
 		mpd_command_list_end(connection);
 
-		list = filelist_new_recv(connection);
+		auto list = filelist_new_recv(connection);
 		list->RemoveDuplicateSongs();
+		return list;
 	} else if (table == SEARCH_URI) {
 		mpd_search_db_songs(connection, exact_match);
 		mpd_search_add_uri_constraint(connection, MPD_OPERATOR_DEFAULT,
 					      filter_utf8.c_str());
 		mpd_search_commit(connection);
 
-		list = filelist_new_recv(connection);
+		return filelist_new_recv(connection);
 	} else {
 		mpd_search_db_songs(connection, exact_match);
 		mpd_search_add_tag_constraint(connection, MPD_OPERATOR_DEFAULT,
@@ -213,20 +226,81 @@ search_simple_query(struct mpd_connection *connection, bool exact_match,
 					      filter_utf8.c_str());
 		mpd_search_commit(connection);
 
-		list = filelist_new_recv(connection);
+		return filelist_new_recv(connection);
+	}
+}
+
+#if LIBMPDCLIENT_CHECK_VERSION(2,10,0)
+
+/**
+ * Throws on error.
+ */
+static time_t
+ParseModifiedSince(const char *s)
+{
+	char *endptr;
+	time_t value = strtoul(s, &endptr, 10);
+	if (endptr == s)
+		throw _("Invalid number");
+
+	constexpr time_t MINUTE = 60;
+	constexpr time_t HOUR = 60 * MINUTE;
+	constexpr time_t DAY = 24 * HOUR;
+	constexpr time_t MONTH = 30 * DAY; // TODO: inaccurate
+	constexpr time_t YEAR = 365 * DAY; // TODO: inaccurate
+
+	s = endptr;
+	switch (*s) {
+	case 's':
+		++s;
+		break;
+
+	case 'M':
+		++s;
+		value *= MINUTE;
+		break;
+
+	case 'h':
+		++s;
+		value *= HOUR;
+		break;
+
+	case 'd':
+		++s;
+		value *= DAY;
+		break;
+
+	case 'm':
+		++s;
+		value *= MONTH;
+		break;
+
+	case 'y':
+	case 'Y':
+		++s;
+		value *= YEAR;
+		break;
+
+	default:
+		throw _("Unrecognized suffix");
 	}
 
-	return list;
+	if (*s != '\0')
+		throw _("Unrecognized suffix");
+
+	return time(nullptr) - value;
 }
 
+#endif
+
 /*-----------------------------------------------------------------------
  * NOTE: This code exists to test a new search ui,
  *       Its ugly and MUST be redesigned before the next release!
  *-----------------------------------------------------------------------
  */
-static FileList *
+static std::unique_ptr<FileList>
 search_advanced_query(struct mpd_connection *connection, const char *query)
-{
+try {
 	advanced_search_mode = false;
 	if (strchr(query, ':') == nullptr)
 		return nullptr;
@@ -296,6 +370,12 @@ search_advanced_query(struct mpd_connection *connection, const char *query)
 			mpd_search_add_uri_constraint(connection,
 						      MPD_OPERATOR_DEFAULT,
 						      value.c_str());
+#if LIBMPDCLIENT_CHECK_VERSION(2,10,0)
+		else if (table[i] == SEARCH_MODIFIED)
+			mpd_search_add_modified_since_constraint(connection,
+								 MPD_OPERATOR_DEFAULT,
+								 ParseModifiedSince(value.c_str()));
+#endif
 		else
 			mpd_search_add_tag_constraint(connection,
 						      MPD_OPERATOR_DEFAULT,
@@ -304,23 +384,24 @@ search_advanced_query(struct mpd_connection *connection, const char *query)
 	}
 
 	mpd_search_commit(connection);
-	auto *fl = filelist_new_recv(connection);
-	if (!mpd_response_finish(connection)) {
-		delete fl;
-		fl = nullptr;
-	}
+	auto fl = filelist_new_recv(connection);
+	if (!mpd_response_finish(connection))
+		fl.reset();
 
 	return fl;
+} catch (...) {
+	mpd_search_cancel(connection);
+	throw;
 }
 
-static FileList *
+static std::unique_ptr<FileList>
 do_search(struct mpdclient *c, const char *query)
 {
 	auto *connection = c->GetConnection();
 	if (connection == nullptr)
 		return nullptr;
 
-	auto *fl = search_advanced_query(connection, query);
+	auto fl = search_advanced_query(connection, query);
 	if (fl != nullptr)
 		return fl;
 
@@ -344,13 +425,12 @@ SearchPage::Reload(struct mpdclient &c)
 		return;
 
 	lw.EnableCursor();
-	delete filelist;
 	filelist = do_search(&c, pattern.c_str());
 	if (filelist == nullptr)
-		filelist = new FileList();
+		filelist = std::make_unique<FileList>();
 	lw.SetLength(filelist->size());
 
-	screen_browser_sync_highlights(filelist, &c.playlist);
+	screen_browser_sync_highlights(*filelist, c.playlist);
 
 	SetDirty();
 }
@@ -413,7 +493,7 @@ void
 SearchPage::Update(struct mpdclient &c, unsigned events) noexcept
 {
 	if (filelist != nullptr && events & MPD_IDLE_QUEUE) {
-		screen_browser_sync_highlights(filelist, &c.playlist);
+		screen_browser_sync_highlights(*filelist, c.playlist);
 		SetDirty();
 	}
 }
